import { id as mid } from "module";
import { TraceSource } from "@implab/core-amd/log/TraceSource";
import { argumentNotNull } from "@implab/core-amd/safe";
import { getScope, render } from "./render";
import { RenditionBase } from "./RenditionBase";
import { Scope } from "./Scope";
import { Observable } from "../observable";
import { destroy } from "./traits";

const trace = TraceSource.get(mid);

export class WatchRendition<T> extends RenditionBase<Node> {
    private readonly _component: (arg: T) => unknown;

    private _node: Node;

    private readonly _scope = new Scope();

    private readonly _subject: Observable<T>;

    constructor(component: (arg: T) => unknown, subject: Observable<T>) {
        super();
        argumentNotNull(component, "component");

        this._component = component;

        this._subject = subject;

        this._node = document.createComment("WatchRendition placeholder");
    }

    protected _create(attrs: object, children: any[]) {
        const scope = getScope();
        scope.own(this._scope);
        scope.own(this._subject.on({ next: this._onValue }));
    }

    private _onValue = (value: T) =>
        void this._render(value).catch( e => trace.error(e));

    private async _render(value: T) {
        this._scope.clean();
        const [refNode, ...rest] = await render(this._component(value), this._node, "replace", this._scope);
        this._node = refNode;
        this._scope.own(() => rest.forEach(destroy));
    }

    protected _getDomNode() {
        if (!this._node)
            throw new Error("The instance of the widget isn't created");
        return this._node;
    }


}
